﻿/********************************************************
 * Purpose: Read the sequence of Event Sets, and process into EPSD
 * Called from FormMain.cs	
 * Input: 1. Event sequence 2. period length  3. min_sup 
 * Output: dSegment, dCount, dF1
 * Author：Kung-Jiuan Yang   Last Update:2013/06/18 
 ********************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PartialPeriodic
{
    public class CreateEPSD
    { 
        string[] lines;
        int mlen = 0;
        double msup;
        double min_sup;

        int segno = 0;
        Dictionary<string, int> dSeg = new Dictionary<string, int>();
        Dictionary<string, double> dF1 = new Dictionary<string, double>();
        Dictionary<string, int> dCount = new Dictionary<string, int>();

        public CreateEPSD(string[] lines, int mlen, double msup)
        {
            try
            {
                //=======================
                // Set Parameters
                //=======================
                this.lines = lines;
                this.mlen = mlen;
                this.msup = msup;
                //=======================
                // Put event sequence into dSeg dictionary
                //=======================
                CreatePSD();
                //
                //=======================
                // Calculate the count for each event tuple
                //=======================
                CreateCount();
                //
                //=======================
                // Find the frequent 1-pattern
                //=======================
                CreateF1();
                //
            }
            catch (Exception e)
            {
                Console.WriteLine(" Error about generating segments: " + e.ToString());
            }
        }
        //==============================================
        // Return all the information
        //==============================================
        public Dictionary<string, int> getDictSeg()
        {
            return dSeg;
        }
        public Dictionary<string, double> getDictF1()
        {
            return dF1;
        }
        public Dictionary<string, int> getDictCount()
        {
            return dCount;
        }
        public int getSegno()
        {
            return segno;
        }
        public double getMin_Sup()
        {
            return min_sup;
        }

        //==============================================
        // Event sequence --> encoded period Segments
        //==============================================
        public void CreatePSD()
        {
            int lineno=0;
            Dictionary<string, string> dTemp = new Dictionary<string, string>();
            
            Dictionary<string, String> dSegNo = new Dictionary<string, String>();
            string keyStr1 = "";
            foreach (var line in lines)
            {
                string ItemString;
                string postString = line;
                //=======================
                //Check the event sets with {}
                //=======================
                if (line.Substring(0, 1) == "{")
                { postString = line.Substring(1, line.Length - 2); }
                int nItem = 0;
                int eventcnt = 0;
                int posno = lineno % mlen + 1;

                if (posno == 1)
                {
                    segno++;
                }

                int findpos = -1;
                do
                {
                    eventcnt++;
                    findpos = postString.IndexOf(",");
                    if (findpos >= 0)
                        ItemString = postString.Substring(0, findpos);
                    else
                        ItemString = postString.Substring(0);
                    nItem = (Int32.Parse(ItemString));
                    string getCode = string.Format("{0:000}", nItem);


                    keyStr1 = string.Format("{0:00}", posno) + getCode + segno.ToString();
                    dSeg.Add(keyStr1, segno);

                    if (findpos >= 0)
                    {
                        int oldfindpos = findpos;
                        postString = postString.Substring(oldfindpos + 1);
                    }
                }
                while (findpos > 0);

                lineno++;
            }
        }
        //==============================================
        // Accumulate the count for each event tuple
        //==============================================
        public void CreateCount()
        {
            foreach (KeyValuePair<string, int> pairS in dSeg)
            {
                string element = pairS.Key.Substring(0, 5);

                if (dCount.ContainsKey(element))
                {
                    dCount[element]++;
                }
                else
                {
                    dCount.Add(element, 1);
                }
            }
           
        }
        //==============================================
        // Write the dF1 if it greater than or equal to min_sup
        //==============================================
        public void CreateF1()
        {
            foreach (KeyValuePair<string, int> pair in dCount)
            {
                min_sup = (double)msup / 100 * segno;
                if (pair.Value >= min_sup)
                {
                    dF1.Add(pair.Key, pair.Value);
                }
            }
        }
    }
}
