﻿/********************************************************
 * Algorithm for Finding Partial Peridoic Pattern
 * Screen Input & Main Program
 * Author：Kung-Jiuan Yang   Last Update:2013/06/18
 *******************************************************/
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PartialPeriodic
{
   
    public partial class FormMain : Form
    {
        static int mlen;
        static double msup;
        static string[] lines;
        int segno = 0;
        double min_sup = 0;
        string path = "c:\\temp\\";

        public FormMain()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //===========================
            // Set Console.WriteLine to a file
            //===========================
            using (StreamWriter writer = new StreamWriter(path+txtOutFN.Text))
            {
                Console.SetOut(writer);
                btnResult.Enabled = false;
                lblStatus.Text = "Running....";
                startProcess();
                lblStatus.Text = lblStatus.Text + "  ==> Done ! ";
                btnResult.Enabled = true;
            }

        }

        private void startProcess()
        {
            Dictionary<string, int> dSeg = new Dictionary<string, int>();
            Dictionary<string, double> dF1 = new Dictionary<string, double>();
            Dictionary<string, int> dCount = new Dictionary<string, int>();

            //============================
            //screen input parameters
            //============================
            double.TryParse(textMinConf.Text, out msup);   // minimum support threshold
            int.TryParse(textPatLen.Text, out mlen);       // periodi length
            lines = System.IO.File.ReadAllLines(@path+cbxInputFN.Text);    //in bin\debug directory
            Console.WriteLine("=========================================");
            Console.WriteLine("Period Length is :" + mlen);
            Console.WriteLine("Min_support :" + msup + "%");
            Console.WriteLine("Input filename: " + path+cbxInputFN.Text);
            Console.WriteLine("Output filename: " + path + txtOutFN.Text);

            //============================
            //Record the starting time & Memory
            //============================
            DateTime StartMoment = DateTime.Now;
            long StartMemory = GC.GetTotalMemory(true);

            //============================
            //1. read the sequence into period segments, and get F1 information
            //============================
            CreateEPSD cepsd = new CreateEPSD(lines, mlen, msup);
            dSeg = cepsd.getDictSeg();
            dF1 = cepsd.getDictF1();
            segno = cepsd.getSegno();
            min_sup = (double)msup / 100 * segno;

            //============================
            //Write out the information
            //============================
            Console.WriteLine("F-1 Count: " + dF1.Count);
            Console.WriteLine("Segment Count: " + segno);
            Console.WriteLine("Algorithm is : " + rbPRA.Text);

            //============================
            //2. Start PRA Main Algorithm
            //   Derive projection-based partial periodic patterns
            //============================
            Dictionary<string, int> dPattern = new Dictionary<string, int>();
            PRAMain pra = new PRAMain(min_sup, segno, dSeg, dF1);
            dPattern = pra.getDictPattern();
            int candcnt = pra.getCandCount();

            //============================
            //Write out the information
            //============================
            Console.WriteLine("Output Pattern Count: " + dPattern.Count);
            Console.WriteLine("Candidate Count: " + candcnt);
            Console.WriteLine("-----------------------------------");

            //============================
            //3. Write out Pattern list
            //   Ouput the patterns with '*' format
            //============================
            CreatePattern cptn = new CreatePattern(mlen, min_sup, segno, dPattern);
            

            //============================
            //Write out the information
            //============================
            Console.WriteLine("-----------------------------------");
            System.DateTime EndMoment = System.DateTime.Now;
            Console.WriteLine("Start Time: " + StartMoment);
            Console.WriteLine("End Time: " + EndMoment);
            string result2 = ((TimeSpan)(EndMoment - StartMoment)).TotalMilliseconds.ToString();
            Console.WriteLine("Execute milliseconds: " + result2);
            long EndMemory = GC.GetTotalMemory(true);
            Console.WriteLine("Memory Used: " + (EndMemory - StartMemory));
            Console.WriteLine("=========================================");
        }

        private void button2_Click_1(object sender, EventArgs e)
        {
            this.Close();
        }
      
         private void btnResult_Click(object sender, EventArgs e)
        {
            lblStatus.Text = "";
            System.Diagnostics.Process.Start("notepad.exe", path+txtOutFN.Text);
        }

       
    }
}
